#!/bin/sh

DEBUG=1
ISMPY_PATH=$HOME/.ismpy
PYTHON=python3
modules="Flask PyYAML numpy matplotlib h5py==2.8.0 Flask-Caching cheroot"
# modules="Flask-Caching cheroot"

usage() {
    cat << EOF

usage: sh install_ismpy.sh [-d]

    -d causes install_ismpy.sh to uninstall ismpy if installed.

EOF
}

info_distro_packages() {
    cat << EOF

---------------------------
We will install the required python packages for you.

However, to avoid the risk of potential installation problems, ***WE STRONGLY
RECOMMEND*** that you install beforehand the following packages through your
os packages manager:

    - numpy
    - matplotib
    - libhdf5

Example

with debian:

    apt-get install python3-numpy
    apt-get install python3-matplotlib
    apt-get install libhdf5-dev


with macports (assuming you use macport's python38)

    ports install py38-numpy
    ports install py38-matplotlib
    ports install hdf5

--------------------------

EOF

}


check_pydev(){
    local _include_py
    $PYTHON -c "import distutils.sysconfig"  > /dev/null  2>&1 
    if [ $? -ne 0 ]; then
        echo "ERROR: distutils.sysconfig not usable" >&2
        exit 2
    fi
    
    _include_py=$($PYTHON -c "from distutils import sysconfig as s;print(s.get_config_vars()['INCLUDEPY'])")
    if [ ! -f "${_include_py}/Python.h" ]; then
        echo "ERROR: python-devel not installed" >&2
        exit 3
    fi
}


check_pyvenv(){
    python3 -m venv -h > /dev/null
    if [ $? -ne 0 ]; then
        echo >&2 "ERROR: The python virtualenv module is not installed, quitting..."
        exit 1
    fi
}


uninstall() {
    [ ! -d $ISMPY_PATH ] && echo "ismpy not installed" && return
    echo "uninstalling ismpy..."
    rm -rf $ISMPY_PATH
    echo "done"
}

[ $# -gt 1 -o "$1" = "-h" ] && usage && exit


which $PYTHON > /dev/null 2>&1
if [ $? -ne 0 ]; then
    echo "ERROR: $PYTHON is not installed or not in the PATH, install it and re-try"
    exit 1
fi
 

if [ "$1" = "-d" ]; then
    uninstall
    exit
fi


if [ ! -d  $ISMPY_PATH ]; then
    echo " "
    echo "ismpy not installed"

    info_distro_packages
    echo -n "Do you want to continue ? (y/n) "
    read answer
    if [ "$answer" = "${answer#[Yy]}" ] ;then
        echo "quitting..."
        exit
    fi

    error=0
    check_pydev
    check_pyvenv

    echo "installing ismpy in $ISMPY_PATH..."
    # bug in python 3.5
    if [ $(expr "$($PYTHON --version)" : ".*3.5") -gt 0 ]; then
        echo "mitigating a bug in python3.5 venv..."
        $PYTHON -m venv $ISMPY_PATH
        $PYTHON -m venv --system-site-packages $ISMPY_PATH
    else
        # force specific package version
        # pip install 'xkcdpass==1.2.5' --force-reinstall
        $PYTHON -m venv --system-site-packages $ISMPY_PATH
    fi

    error=$(( $error + $? ))

    echo "upgrading pip to bypass some distros pip bug..."
    $ISMPY_PATH/bin/python3 -m pip install --upgrade pip

    echo -n "installing ismpy modules..."
    for m in $modules; do
        printf "\n  %s" "installing $m..."
        if [ $DEBUG -eq 1 ]; then
            $ISMPY_PATH/bin/pip install --upgrade-strategy "only-if-needed" $m
        else
            $ISMPY_PATH/bin/pip install $m > /dev/null 2>&1
        fi

        if [ $? -eq 0 ]; then
            printf "%s" "ok"
        else
            printf "%s" "nok"
            error=$(( $error + 1 ))
        fi

    done
    echo ""

    if [ $error -eq 0 ]; then
        printf "\n\n\n%s\n  %s\n\n" "you can now launch the ISM python softwares with the following python instance:" "$ISMPY_PATH/bin/$PYTHON"
    else
        uninstall
        echo "sorry, an error occured during the installation of ismpy"
    fi
fi

